@@ -1,6 +1,8 @@
 from PIL import Image, ImageDraw
 from collections import defaultdict
 import copy
+import operator
+from typing import Union
 
 from .util import coordgen, nested_int_loop
 
@@ -66,7 +68,7 @@ class ImageLayer:
         return self.toimage().save(*args, **kwargs)
     
     def toimage(self):
-        img = Image.new('RGBA', self.image.size, color=(0,0,0,255))
+        img = Image.new(self.image.mode, self.image.size, color=(0,0,0,255))
         px = img.load()
         for x, y in self.pxcs:
             px[x,y] = self.color
@@ -89,6 +91,7 @@ class OptimalColors:
         self.colors = colors
 
     def adjustpixelrgb(self, pixel, limit=1, shift=0):
+        print(pixel)
         r,g,b = pixel
         for mr,mg,mb in self.colors:
             if mr - limit + shift < r < mr + limit + shift and \
@@ -111,11 +114,11 @@ class OptimalColors:
         for bw in self.colors:
             if bw - limit + shift < pixel < bw + limit + shift:
                 return bw
+        return pixel
     
     def optimize_image(self, image, tolerance=1, bwshift=0):
         px = image.load()
-        for x in range(image.width):
-            for y in range(image.height):
+        for x,y in coordgen(*image.size):
                 try:
                     px[x,y] = self.adjustpixelbw(px[x,y], tolerance, bwshift)
                 except TypeError:
@@ -123,7 +126,38 @@ class OptimalColors:
                         px[x,y] = self.adjustpixelrgb(px[x,y], tolerance, bwshift)
                     except ValueError:
                         px[x,y] = self.adjustpixelrgba(px[x,y], tolerance, bwshift)
+                
 
+class ColorVector:
+    def __init__(self, color: Union[int, tuple]):
+        if type(color) == int:
+            self.color = (color,)
+        else:
+            self.color = color
+    
+    def __add__(self, other):
+        return self.op(other, operator.add)
+    
+    def __sub__(self, other):
+        return self.op(other, operator.sub)
+    
+    def op(self, other, op):
+        return ColorVector([op(a,b) for a,b in zip(self.color, other.color)])
+
+    def function(self, func):
+        return ColorVector([func(a) for a in zip(self.color)])
+
+    def size(self):
+        def distance(*args):
+            if len(args) > 2:
+                a, b, *c = args
+                return distance((a**2+b**2)**0.5, *c)
+            elif len(args) == 2:
+                a, b = args
+                return (a**2+b**2)**0.5
+            elif len(args) == 1:
+                return abs(args[0])
+        return distance(*self.color)
 
 class LayeredImage:
     def __init__(self, image):
@@ -148,8 +182,7 @@ class LayeredImage:
         target_color = get_target_color(self)
         px = orig.load()
         pixels = set()
-        for x in range(orig.width):
-            for y in range(orig.height):
+        for x,y in coordgen(*orig.size):
                 if (x,y) in pixels:
                     continue
                 ImageDraw.floodfill(orig, xy=(x,y), value=target_color)
@@ -165,7 +198,14 @@ class LayeredImage:
                 orig = self.image.copy()
                 px = orig.load()
         
-    def optimize_color_boundaries(self, colors: tuple):
+    def optimize_color_boundaries(self, colors: tuple, *args, **kwargs):
         optcolors = OptimalColors(colors)
-        optcolors.optimize_image(self.image)
+        optcolors.optimize_image(self.image, *args, **kwargs)
 
+    def strictcolor(self, colors: tuple):
+        px = self.image.load()
+        def colordistance(a):
+            return (ColorVector(px[x,y]) - ColorVector(a)).size()
+        for x, y in coordgen(*self.image.size):
+            choices = sorted(colors, key=lambda x: colordistance(x))
+            px[x,y] = choices[0]
\ No newline at end of file
